#![no_main]
#![no_std]

use algorithms::crc::{Crc, Crc32 as Crc32Rust, ffi::Crc32 as Crc32C};

use defmt::{debug, error, info, println, trace, warn};
use panic_probe as _;

use defmt_rtt as _; // global logger

use cortex_m_rt::entry;
use hal::{pac, prelude::*, rcc::Config};

// same panicking *behavior* as `panic-probe` but doesn't print a panic message
// this prevents the panic message being printed *twice* when `defmt::panic` is invoked
#[defmt::panic_handler]
fn panic() -> ! {
    cortex_m::asm::udf()
}

const DATA: &[u8] = b"hello world AAAAAAAAAAAAAAAAAAAAAAA";

#[entry]
fn main() -> ! {
    let dp = pac::Peripherals::take().unwrap();
    let cp = cortex_m::Peripherals::take().unwrap();

    // Configure the clock.
    let mut rcc = dp.RCC.freeze(Config::hsi16());

    // Acquire the GPIOA peripheral. This also enables the clock for GPIOA in
    // the RCC register.
    let gpioa = dp.GPIOA.split(&mut rcc);

    // Configure PA5 as output.
    let mut led = gpioa.pa5.into_push_pull_output();

    // Get the delay provider.
    let mut delay = cp.SYST.delay(rcc.clocks);

    let mut crc;

    #[allow(clippy::never_loop)]
    loop {
        println!("Hello World!");
        trace!("trace log");
        debug!("debug log");
        info!("info log");
        warn!("warn log");
        error!("error log");

        led.set_high().unwrap();
        delay.delay_ms(500_u16);

        crc = Crc32Rust::checksum(DATA);
        info!("CRC (Rust): {:08x}", crc);
        crc = Crc32C::checksum(DATA);
        info!("CRC (C)   : {:08x}", crc);

        led.set_low().unwrap();
        delay.delay_ms(500_u16);
    }
}
